package com.mashangban.example.controller;

import java.io.IOException;
import java.util.Date;
import java.util.Random;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSONObject;
import com.mashangban.example.tool.OpenApiInvoke;
import com.mashangban.example.utils.CookieUtil;
import com.mashangban.example.utils.HttpClientFactory;
import com.mashangban.example.utils.JssdkUtil;

@Controller
@RequestMapping("/msb/*")
public class MsbController {
	
	private static final Logger logger = LoggerFactory.getLogger(MsbController.class);
	
	private static final String USER_MSB_LOGIN_KEY = "_user_msb_login_key";
	
	/**
	 * 免登流程入口地址，对应自建应用的应用入口地址，比如url：https://oa.com/msb/auth。
	*/
	@RequestMapping("auth")
	public void auth(HttpServletRequest request,HttpServletResponse response) {
		logger.info("入口请求...");
		String userLoginKey = CookieUtil.getCookieValue(request, USER_MSB_LOGIN_KEY);
		try {
			if(checkLoginInfo(userLoginKey, request)) {
				//如果已经存在了用户登录过的信息，说明可以不走认证了，直接跳转到企业应用相应界面即可，这样可以加快访问的速度。
				logger.info("已存在相关登录过的信息，建议直接转向企业应用相关界面即可...");
				response.sendRedirect(OpenApiInvoke.RETURN_URL);
			}else {
				//需要认证，则拼接参数直接重定向到oauth的地址，特别注意跳转回的企业应用接收地址，比如这里是OpenApiInvoke.RETURN_URL
				String redirectUrl = OpenApiInvoke.OAUTH_HOST + "/authorize?response_type=code" 
				+ "&client_id="+ OpenApiInvoke.appKey 
				+ "&state="+"&redirect_uri=" 
				+ OpenApiInvoke.urlEncode(OpenApiInvoke.RETURN_URL);
				response.sendRedirect(redirectUrl);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 上一步msb/auth后的回调地址，对应企业应用的登录入口。比如url：https://oa.com/msb/login
	*/
	@RequestMapping("login")
	public String login(HttpServletRequest request,HttpServletResponse response,String code) {
		logger.info("跳转到登录请求...");
		String userLoginKey = CookieUtil.getCookieValue(request, USER_MSB_LOGIN_KEY);
		//是否需要请求马上办接口
		boolean needAuth = true;
		try {
			//可以先检查是否已存在登录过的信息
			if(checkLoginInfo(userLoginKey, request)) {
				//企业应用已保存登录信息，可不走认证流程
				needAuth = false;
			}
			//如果需要进行认证
			if(needAuth) {
				//拼接认证需要的url和参数
				String requestUrl = OpenApiInvoke.OAUTH_HOST + "/token?grant_type=authorization_code"+"&code="+code+"&client_id="+OpenApiInvoke.appKey+"&client_secret="+OpenApiInvoke.appSecret;
				//{"openid":"xxx","corpOpenid":"xx"}
				String result = HttpClientFactory.post(requestUrl, null, null);
				if(StringUtils.isNotEmpty(result)) {
					JSONObject obj = JSONObject.parseObject(result);
					String openId = obj.getString("openid");
					if(StringUtils.isNotEmpty(openId)) {
						//--------------------请根据实际业务调整--------------------
						//TODO企业应用可以根据openid转换为对应的用户，然后进行登录操作。成功后设置缓存，cookie等。这样这套免登认证就完成了。
						logger.info("认证成功，做相关登录操作，以下为示例流程...");
						//TODO例如，登录成功后设置用户的cookie信息，生成一个uuid，将uuid和openid的对应起来，存放在缓存中等。
						String uuid = UUID.randomUUID().toString();
						//TODO存放在session中，建议改用缓存中间件服务，比如redis
						request.getSession().setAttribute(uuid, openId);
						userLoginKey = uuid;
						String sessionOpenId = openId;
						//TODO设置cookie信息
						CookieUtil.addCookie(request, response, USER_MSB_LOGIN_KEY, userLoginKey, CookieUtil.ONE_DAY);
						//TODO例如，跳转到welcome页面
						request.setAttribute("userLoginKey", userLoginKey);
						request.setAttribute("openId", sessionOpenId);
						return "welcome";
						//-------------------------------------------------------
					}
				}
			} else {
				logger.info("已存在登录信息，直接转向企业相关页面...");
				//TODO以下跳转页面请根据企业应用自行调整
				String sessionOpenId = (String) request.getSession().getAttribute(userLoginKey);
				request.setAttribute("userLoginKey", userLoginKey);
				request.setAttribute("openId", sessionOpenId);
				return "welcome";
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		//其他情况为认证不通过的跳转页面。
		return "error";
	}
	
	private boolean checkLoginInfo(String userLoginKey,HttpServletRequest request) {
		boolean flag = false;
		if(StringUtils.isNotEmpty(userLoginKey)) {
			String sessionOpenId = (String) request.getSession().getAttribute(userLoginKey);
			if(StringUtils.isNotEmpty(sessionOpenId)) {
				//已缓存了用户登录的信息
				flag = true;
			}
		}
		return flag;
	}
	
	
	@RequestMapping("getToken")
	@ResponseBody
	public String getToken() {
		logger.info("获取accessToken...");
		String result = OpenApiInvoke.getAccessToken(OpenApiInvoke.appKey, OpenApiInvoke.appSecret, OpenApiInvoke.permAuth);
		return result;
	}
	
	@RequestMapping("getDepartmentList")
	@ResponseBody
	public String getDepartmentList(String accessToken) {
		logger.info("获取部门信息...");
		JSONObject obj = new JSONObject();
		//TODO自行调整参数值
		obj.put("id", "0");
		obj.put("hasAllChild", 1);
		String result = OpenApiInvoke.getDepartmentList(accessToken,obj.toJSONString());
		return result;
	}
	
	@RequestMapping("getContactUserList")
	@ResponseBody
	public String getContactUserList(String accessToken) {
		logger.info("获取部门成员信息...");
		JSONObject obj = new JSONObject();
		//TODO自行调整参数值
		obj.put("departmentId", 43974);
		String result = OpenApiInvoke.getContactUserList(accessToken,obj.toJSONString());
		return result;
	}
	
	@RequestMapping("sendMsg")
	@ResponseBody
	public String sendMsg(String accessToken) {
		logger.info("发送消息...");
		JSONObject obj = new JSONObject();
		//TODO调整正确的接收者参数值
		obj.put("to", "UUFSGmKgI+8=");
		obj.put("type", "numen");
		JSONObject body = new JSONObject();
		body.put("title","这是一个单图文的测试消息");
		body.put("desc","这里可以定制内容描述");
		body.put("img","https://nos.netease.com/kolibri/image/2.7.5.png");
		body.put("mobileUrl","https://qiye.yixin.im");
		body.put("pcUrl","https://qiye.yixin.im");
		obj.put("body", body);
		String result = OpenApiInvoke.sendMsg(accessToken,obj.toJSONString());
		return result;
	}
	
	@RequestMapping("getJssdkTicket")
	@ResponseBody
	public String getJssdkTicket(String accessToken) {
		logger.info("获取JssdkTicket...");
		String result = OpenApiInvoke.getJssdkTicket(accessToken);
		return result;
	}
	
	@RequestMapping("getJssdkSign")
	@ResponseBody
	public String getJssdkSign(String jssdkTicket) {
		logger.info("生成jssdk的签名...");
		//获取随机字符串
		String nonce = genRandomWord(6);
		Date now = new Date();
		String timestamp = String.valueOf(now.getTime());
		//TODO根据实际情况调整url，设置为需要使用jssdk的页面url
		String url = "https://oa.com/xxxx";
		//生成签名
		String sign = JssdkUtil.generateJssdkSignature(nonce, timestamp, jssdkTicket, url);
		JSONObject result = new JSONObject();
		result.put("nonce", nonce);
		result.put("timestamp", timestamp);
		result.put("url", url);
		result.put("sign", sign);
		return result.toJSONString();
	}
	
	private String genRandomWord(int len) {
		char[] str = { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y',
				'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
		return genRandom(len, str);
	}
	
	private  String genRandom(int len,char[] arr) {
		int maxNum = arr.length;
		// 生成的随机数
		int i;
		// 生成的长度
		int count = 0;
		
		StringBuffer pwd = new StringBuffer("");
		Random r = new Random();
		while (count < len) {
			// 生成随机数，取绝对值，防止生成负数，
			i = Math.abs(r.nextInt(maxNum));
			if (i >= 0 && i < arr.length) {
				pwd.append(arr[i]);
				count++;
			}
		}
		return pwd.toString();
	}
}
