package com.mashangban.example.utils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HttpClientFactory {

	private static final Logger logger = LoggerFactory.getLogger(HttpClientFactory.class);

	public static final String APPLICATION_FORM_URLENCODED = "application/x-www-form-urlencoded;charset=utf-8";

	public static final String METHOD_POST = "POST";
	public static final String METHOD_GET = "GET";

	private static final int maxConnection = 1024;
	private static final int CONNECTION_TIMEOUT = 3000;
	private static final int SO_TIMEOUT = 5000;
	private static final String reqCharset = "UTF-8";
	private static final String resCharset = "UTF-8";

	private static HttpClientUtil httpClientUtil;

	static {
		if (null == httpClientUtil) {
			httpClientUtil = new HttpClientUtil(maxConnection, CONNECTION_TIMEOUT, SO_TIMEOUT, true);
			logger.debug("new HttpClientUtil");
		}
	}

	public static HttpClientUtil getHttpClientUtil() {
		return httpClientUtil;
	}

	public static HttpClient getHttpClient() {
		return httpClientUtil.getHttpClient();
	}

	/**
	 * 模拟表单post提交，表单参数存入params,针对大数据量结果返回可以不输出日志
	 */
	public static String post(String url, Map<String, String> params, Map<String, String> headers, boolean needLogger) throws ClientProtocolException, IOException {

		HttpPost post = new HttpPost(url);
		try {
			if (null != params) {
				List<NameValuePair> nvps = new ArrayList<NameValuePair>();
				for (Map.Entry<String, String> entry : params.entrySet()) {
					nvps.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
				}
				post.setEntity(new UrlEncodedFormEntity(nvps, reqCharset));
			}

			return returnResponse(headers, post, url, needLogger);

		} finally {
			post.releaseConnection();
		}
	}

	/**
	 * 模拟表单post提交，表单参数存入params
	 * 
	 * @param url
	 * @param params
	 * @param headers
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	public static String post(String url, Map<String, String> params, Map<String, String> headers) throws ClientProtocolException, IOException {

		return post(url, params, headers, true);
	}

	/**
	 * get请求
	 * 
	 * @param url
	 * @param headers
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	public static String get(String url, Map<String, String> headers) throws ClientProtocolException, IOException {

		HttpGet get = new HttpGet(url);
		try {
			return returnResponse(headers, get, url);
		} finally {
			get.releaseConnection();
		}
	}

	/**
	 * 模拟body方式提交，对应springmvc的@RequestBody
	 * 
	 * @param url
	 * @param params
	 * @param headers
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	public static String postBody(String url, String params, Map<String, String> headers) throws ClientProtocolException, IOException {
		logRequest(url, "POST", params, null);
		HttpPost post = new HttpPost(url);
		try {
			if (StringUtils.isNotEmpty(params)) {
				StringEntity str = new StringEntity(params, reqCharset);
				str.setContentEncoding(reqCharset);
				str.setContentType("application/json");
				post.setEntity(str);
			}
			return returnResponse(headers, post, url);

		} finally {
			post.releaseConnection();
		}
	}

	/**
	 * 封装公共返回处理
	 * 
	 * @param headers
	 * @param url
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	private static String returnResponse(Map<String, String> headers, HttpUriRequest request, String url) throws ClientProtocolException, IOException {
		return returnResponse(headers, request, url, true);
	}

	/**
	 * 封装公共返回处理
	 * 
	 * @param headers
	 * @param url
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	private static String returnResponse(Map<String, String> headers, HttpUriRequest request, String url, boolean needLogger) throws ClientProtocolException, IOException {
		if (headers != null) {
			for (Map.Entry<String, String> entry : headers.entrySet()) {
				request.addHeader(entry.getKey(), entry.getValue());
			}
		}
		long start = System.currentTimeMillis();

		HttpResponse response = null;
		try {
			response = getHttpClient().execute(request);
			StatusLine sl = response.getStatusLine();
			String result = null;
			if (sl.getStatusCode() == HttpStatus.SC_OK) {
				HttpEntity entity = response.getEntity();
				result = EntityUtils.toString(entity, resCharset);
				if (logger.isDebugEnabled() && needLogger) {
					logger.debug(" requestUrl {} ,returnResponse {}  {} ms", url, result, System.currentTimeMillis() - start);
				}
				return result;
			} else {
				logger.error("req url failed, url: " + url + " ,retcode: " + sl.getStatusCode());
				EntityUtils.consume(response.getEntity());
			}
		} catch (ClientProtocolException e) {
			logger.error(e.getMessage(), e);
			request.abort();
			throw e;
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			request.abort();
			throw e;
		} finally {
			if (response != null) {
				EntityUtils.consumeQuietly(response.getEntity());
			}
		}

		return null;
	}

	public static void logRequest(String url, String method, final String params, String body) {
		logger.info("HTTPRequest:[U]:{},[M]:{},[P]:{},[B]:{}", url, method, params, body);
	}

}
